@@ -13,18 +13,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import base64
 import logging
 
 from os_ken.ofproto import ether
-from os_ken.ofproto import ofproto_v1_4
-from os_ken.ofproto import ofproto_v1_4_parser
+from os_ken.ofproto import ofproto_v1_5
+from os_ken.ofproto import ofproto_v1_5_parser
 from os_ken.lib import ofctl_utils
 
 LOG = logging.getLogger(__name__)
 
 DEFAULT_TIMEOUT = 1.0
 
-UTIL = ofctl_utils.OFCtlUtil(ofproto_v1_4)
+UTIL = ofctl_utils.OFCtlUtil(ofproto_v1_5)
 str_to_int = ofctl_utils.str_to_int
 
 
@@ -79,9 +80,6 @@ def to_instructions(dp, insts):
             instructions.append(
                 parser.OFPInstructionWriteMetadata(
                     metadata, metadata_mask))
-        elif inst_type == 'METER':
-            meter_id = str_to_int(i.get('meter_id'))
-            instructions.append(parser.OFPInstructionMeter(meter_id))
         else:
             LOG.error('Unknown instruction type: %s', inst_type)
 
@@ -93,11 +91,15 @@ def action_to_str(act):
     t = UTIL.ofp_action_type_to_user(s['type'])
     s['type'] = t if t != s['type'] else 'UNKNOWN'
 
-    if 'field' in s:
+    if t == 'SET_FIELD':
         field = s.pop('field')
         s['field'] = field['OXMTlv']['field']
         s['mask'] = field['OXMTlv']['mask']
         s['value'] = field['OXMTlv']['value']
+    elif t == 'COPY_FIELD':
+        oxm_ids = s.pop('oxm_ids')
+        s['src_oxm_id'] = oxm_ids[0]['OFPOxmId']['type']
+        s['dst_oxm_id'] = oxm_ids[1]['OFPOxmId']['type']
 
     return s
 
@@ -111,7 +113,7 @@ def instructions_to_str(instructions):
         t = UTIL.ofp_instruction_type_to_user(v['type'])
         inst_type = t if t != v['type'] else 'UNKNOWN'
         # apply/write/clear-action instruction
-        if isinstance(i, ofproto_v1_4_parser.OFPInstructionActions):
+        if isinstance(i, ofproto_v1_5_parser.OFPInstructionActions):
             acts = []
             for a in i.actions:
                 acts.append(action_to_str(a))
@@ -167,7 +169,10 @@ def to_match(dp, attrs):
                'pbb_isid': ofctl_utils.to_match_masked_int,
                'tunnel_id': ofctl_utils.to_match_masked_int,
                'ipv6_exthdr': ofctl_utils.to_match_masked_int,
-               'pbb_uca': str_to_int}
+               'pbb_uca': str_to_int,
+               'tcp_flags': str_to_int,
+               'actset_output': str_to_int,
+               'packet_type': ofctl_utils.to_match_packet_type}
 
     keys = {'dl_dst': 'eth_dst',
             'dl_src': 'eth_src',
@@ -200,7 +205,7 @@ def to_match(dp, attrs):
 
 
 def to_match_vid(value):
-    return ofctl_utils.to_match_vid(value, ofproto_v1_4.OFPVID_PRESENT)
+    return ofctl_utils.to_match_vid(value, ofproto_v1_5.OFPVID_PRESENT)
 
 
 def match_to_str(ofmatch):
@@ -217,6 +222,8 @@ def match_to_str(ofmatch):
             value = match_vid_to_str(value, mask)
         elif key == 'in_port':
             value = UTIL.ofp_port_to_user(value)
+        elif key == 'packet_type':
+            value = [value >> 16, value & 0xffff]
         else:
             if mask is not None:
                 value = str(value) + '/' + str(mask)
@@ -227,7 +234,7 @@ def match_to_str(ofmatch):
 
 def match_vid_to_str(value, mask):
     return ofctl_utils.match_vid_to_str(
-        value, mask, ofproto_v1_4.OFPVID_PRESENT)
+        value, mask, ofproto_v1_5.OFPVID_PRESENT)
 
 
 def wrap_dpid_dict(dp, value, to_user=True):
@@ -237,6 +244,31 @@ def wrap_dpid_dict(dp, value, to_user=True):
     return {dp.id: value}
 
 
+def stats_to_str(ofstats):
+
+    stats = {}
+    ofstats = ofstats.to_jsondict()['OFPStats']
+    ofstats = ofstats['oxs_fields']
+
+    for s in ofstats:
+        key = s['OXSTlv']['field']
+        if key == 'duration':
+            value = {
+                'duration_sec': s['OXSTlv']['value'][0],
+                'duration_nsec': s['OXSTlv']['value'][1],
+            }
+        elif key == 'idle_time':
+            value = {
+                'idle_time_sec': s['OXSTlv']['value'][0],
+                'idle_time_nsec': s['OXSTlv']['value'][1],
+            }
+        else:
+            value = s['OXSTlv']['value']
+        stats.setdefault(key, value)
+
+    return stats
+
+
 def get_desc_stats(dp, waiters, to_user=True):
     stats = dp.ofproto_parser.OFPDescStatsRequest(dp, 0)
     msgs = []
@@ -315,6 +347,44 @@ def get_queue_desc(dp, waiters, port_no=None, queue_id=None, to_user=True):
     return wrap_dpid_dict(dp, configs, to_user)
 
 
+def get_flow_desc_stats(dp, waiters, flow=None, to_user=True):
+    flow = flow if flow else {}
+    table_id = UTIL.ofp_table_from_user(
+        flow.get('table_id', dp.ofproto.OFPTT_ALL))
+    flags = str_to_int(flow.get('flags', 0))
+    out_port = UTIL.ofp_port_from_user(
+        flow.get('out_port', dp.ofproto.OFPP_ANY))
+    out_group = UTIL.ofp_group_from_user(
+        flow.get('out_group', dp.ofproto.OFPG_ANY))
+    cookie = str_to_int(flow.get('cookie', 0))
+    cookie_mask = str_to_int(flow.get('cookie_mask', 0))
+    match = to_match(dp, flow.get('match', {}))
+    # Note: OpenFlow does not allow to filter flow entries by priority,
+    # but for efficiency, ofctl provides the way to do it.
+    priority = str_to_int(flow.get('priority', -1))
+
+    stats = dp.ofproto_parser.OFPFlowDescStatsRequest(
+        dp, flags, table_id, out_port, out_group, cookie, cookie_mask,
+        match)
+
+    msgs = []
+    ofctl_utils.send_stats_request(dp, stats, waiters, msgs, LOG)
+
+    flows = []
+    for msg in msgs:
+        for stats in msg.body:
+            if 0 <= priority != stats.priority:
+                continue
+
+            s = stats.to_jsondict()[stats.__class__.__name__]
+            s['instructions'] = instructions_to_str(stats.instructions)
+            s['stats'] = stats_to_str(stats.stats)
+            s['match'] = match_to_str(stats.match)
+            flows.append(s)
+
+    return wrap_dpid_dict(dp, flows, to_user)
+
+
 def get_flow_stats(dp, waiters, flow=None, to_user=True):
     flow = flow if flow else {}
     table_id = UTIL.ofp_table_from_user(
@@ -345,7 +415,7 @@ def get_flow_stats(dp, waiters, flow=None, to_user=True):
                 continue
 
             s = stats.to_jsondict()[stats.__class__.__name__]
-            s['instructions'] = instructions_to_str(stats.instructions)
+            s['stats'] = stats_to_str(stats.stats)
             s['match'] = match_to_str(stats.match)
             flows.append(s)
 
@@ -376,6 +446,7 @@ def get_aggregate_flow_stats(dp, waiters, flow=None, to_user=True):
     for msg in msgs:
         stats = msg.body
         s = stats.to_jsondict()[stats.__class__.__name__]
+        s['stats'] = stats_to_str(stats.stats)
         flows.append(s)
 
     return wrap_dpid_dict(dp, flows, to_user)
@@ -418,12 +489,18 @@ def get_table_features(dp, waiters, to_user=True):
                       ofproto.OFPTFPT_APPLY_ACTIONS,
                       ofproto.OFPTFPT_APPLY_ACTIONS_MISS]
 
+    p_type_packet = ofproto.OFPTFPT_PACKET_TYPES
+
     p_type_oxms = [ofproto.OFPTFPT_MATCH,
                    ofproto.OFPTFPT_WILDCARDS,
                    ofproto.OFPTFPT_WRITE_SETFIELD,
                    ofproto.OFPTFPT_WRITE_SETFIELD_MISS,
                    ofproto.OFPTFPT_APPLY_SETFIELD,
-                   ofproto.OFPTFPT_APPLY_SETFIELD_MISS]
+                   ofproto.OFPTFPT_APPLY_SETFIELD_MISS,
+                   ofproto.OFPTFPT_WRITE_COPYFIELD,
+                   ofproto.OFPTFPT_WRITE_COPYFIELD_MISS,
+                   ofproto.OFPTFPT_APPLY_COPYFIELD,
+                   ofproto.OFPTFPT_APPLY_COPYFIELD_MISS]
 
     p_type_experimenter = [ofproto.OFPTFPT_EXPERIMENTER,
                            ofproto.OFPTFPT_EXPERIMENTER_MISS]
@@ -462,6 +539,12 @@ def get_table_features(dp, waiters, to_user=True):
                         oxm = i.to_jsondict()[i.__class__.__name__]
                         oxm_ids.append(oxm)
                     p['oxm_ids'] = oxm_ids
+                elif prop.type == p_type_packet:
+                    oxm_values = []
+                    for val in prop.oxm_values:
+                        i = {val[0]: val[1]}
+                        oxm_values.append(i)
+                    p['oxm_values'] = oxm_values
                 elif prop.type in p_type_experimenter:
                     pass
                 properties.append(p)
@@ -582,7 +665,7 @@ def get_meter_features(dp, waiters, to_user=True):
     return wrap_dpid_dict(dp, features, to_user)
 
 
-def get_meter_config(dp, waiters, meter_id=None, to_user=True):
+def get_meter_desc(dp, waiters, meter_id=None, to_user=True):
     flags = {dp.ofproto.OFPMF_KBPS: 'KBPS',
              dp.ofproto.OFPMF_PKTPS: 'PKTPS',
              dp.ofproto.OFPMF_BURST: 'BURST',
@@ -593,7 +676,7 @@ def get_meter_config(dp, waiters, meter_id=None, to_user=True):
     else:
         meter_id = UTIL.ofp_meter_from_user(meter_id)
 
-    stats = dp.ofproto_parser.OFPMeterConfigStatsRequest(
+    stats = dp.ofproto_parser.OFPMeterDescStatsRequest(
         dp, 0, meter_id)
     msgs = []
     ofctl_utils.send_stats_request(dp, stats, waiters, msgs, LOG)
@@ -687,6 +770,8 @@ def get_group_features(dp, waiters, to_user=True):
                    ofp.OFPAT_SET_FIELD: 'SET_FIELD',
                    ofp.OFPAT_PUSH_PBB: 'PUSH_PBB',
                    ofp.OFPAT_POP_PBB: 'POP_PBB',
+                   ofp.OFPAT_COPY_FIELD: 'COPY_FIELD',
+                   ofp.OFPAT_METER: 'METER',
                    ofp.OFPAT_EXPERIMENTER: 'EXPERIMENTER'}
 
     stats = dp.ofproto_parser.OFPGroupFeaturesStatsRequest(dp, 0)
@@ -748,8 +833,13 @@ def get_group_features(dp, waiters, to_user=True):
     return wrap_dpid_dict(dp, features, to_user)
 
 
-def get_group_desc(dp, waiters, to_user=True):
-    stats = dp.ofproto_parser.OFPGroupDescStatsRequest(dp, 0)
+def get_group_desc(dp, waiters, group_id=None, to_user=True):
+    if group_id is None:
+        group_id = dp.ofproto.OFPG_ALL
+    else:
+        group_id = UTIL.ofp_group_from_user(group_id)
+
+    stats = dp.ofproto_parser.OFPGroupDescStatsRequest(dp, 0, group_id)
     msgs = []
     ofctl_utils.send_stats_request(dp, stats, waiters, msgs, LOG)
 
@@ -767,7 +857,14 @@ def get_group_desc(dp, waiters, to_user=True):
 
                     else:
                         actions.append(action)
+                properties = []
+                for prop in bucket.properties:
+                    p = prop.to_jsondict()[prop.__class__.__name__]
+                    t = UTIL.ofp_group_bucket_prop_type_to_user(prop.type)
+                    p['type'] = t if t != prop.type else 'UNKNOWN'
+                    properties.append(p)
                 b['actions'] = actions
+                b['properties'] = properties
                 buckets.append(b)
 
             d['buckets'] = buckets
@@ -842,7 +939,7 @@ def mod_flow_entry(dp, flow, cmd):
     flow_mod = dp.ofproto_parser.OFPFlowMod(
         dp, cookie, cookie_mask, table_id, cmd, idle_timeout,
         hard_timeout, priority, buffer_id, out_port, out_group,
-        flags, importance, match, inst)
+        importance, flags, match, inst)
 
     ofctl_utils.send_msg(dp, flow_mod, LOG)
 
@@ -891,6 +988,9 @@ def mod_meter_entry(dp, meter, cmd):
 
 
 def mod_group_entry(dp, group, cmd):
+    ofp = dp.ofproto
+    parser = dp.ofproto_parser
+
     group_type = str(group.get('type', 'ALL'))
     t = UTIL.ofp_group_type_from_user(group_type)
     group_type = t if t != group_type else None
@@ -898,24 +998,73 @@ def mod_group_entry(dp, group, cmd):
         LOG.error('Unknown group type: %s', group.get('type'))
 
     group_id = UTIL.ofp_group_from_user(group.get('group_id', 0))
+    command_bucket_id = str_to_int(group.get('command_bucket_id', 0))
+
+    # Note:
+    # The list of group property types that are currently defined
+    # are only OFPGPT_EXPERIMENTER(Experimenter defined).
+    properties = []
 
     buckets = []
     for bucket in group.get('buckets', []):
-        weight = str_to_int(bucket.get('weight', 0))
-        watch_port = str_to_int(
-            bucket.get('watch_port', dp.ofproto.OFPP_ANY))
-        watch_group = str_to_int(
-            bucket.get('watch_group', dp.ofproto.OFPG_ANY))
-        actions = []
+
+        # get bucket_id in buckets
+        bucket_id = str_to_int(bucket.get('bucket_id', 0))
+
+        # get actions in buckets
+        bucket_actions = []
         for dic in bucket.get('actions', []):
             action = to_action(dp, dic)
             if action is not None:
-                actions.append(action)
-        buckets.append(dp.ofproto_parser.OFPBucket(
-            weight, watch_port, watch_group, actions))
+                bucket_actions.append(action)
 
-    group_mod = dp.ofproto_parser.OFPGroupMod(
-        dp, cmd, group_type, group_id, buckets)
+        # get properties in buckets
+        bucket_properties = []
+        for p in bucket.get('properties', []):
+            group_bp_type = str(p.get('type', 'WEIGHT'))
+            t = UTIL.ofp_group_bucket_prop_type_from_user(group_bp_type)
+            group_bp_type = t if t != group_bp_type else ofp.OFPGBPT_WEIGHT
+
+            if group_bp_type == ofp.OFPGBPT_WEIGHT:
+                weight = str_to_int(p.get('weight', 0))
+                bucket_properties.append(
+                    parser.OFPGroupBucketPropWeight(
+                        type_=group_bp_type, weight=weight))
+            elif group_bp_type == ofp.OFPGBPT_WATCH_PORT:
+                watch_port = str_to_int(p.get('watch', dp.ofproto.OFPP_ANY))
+                bucket_properties.append(
+                    parser.OFPGroupBucketPropWatch(
+                        type_=group_bp_type, watch=watch_port))
+            elif group_bp_type == ofp.OFPGBPT_WATCH_GROUP:
+                watch_group = str_to_int(p.get('watch', dp.ofproto.OFPG_ANY))
+                bucket_properties.append(
+                    parser.OFPGroupBucketPropWatch(
+                        type_=group_bp_type, watch=watch_group))
+            elif group_bp_type == ofp.OFPGBPT_EXPERIMENTER:
+                experimenter = p.get('experimenter', 0)
+                exp_type = p.get('exp_type', 0)
+                data_type = p.get('data_type', 'ascii')
+                if data_type not in ['ascii', 'base64']:
+                    LOG.error('Unknown data type: %s', data_type)
+                data = p.get('data', '')
+                if data_type == 'base64':
+                    data = base64.b64decode(data)
+                bucket_properties.append(
+                    parser.OFPGroupBucketPropExperimenter(
+                        type_=group_bp_type, experimenter=experimenter,
+                        exp_type=exp_type, data=data))
+            else:
+                LOG.error('Unknown group bucket prop type: %s', p['type'])
+
+        # create bucket
+        bucket = parser.OFPBucket(bucket_id=bucket_id,
+                                  actions=bucket_actions,
+                                  properties=bucket_properties)
+        buckets.append(bucket)
+
+    group_mod = parser.OFPGroupMod(dp, cmd, group_type, group_id,
+                                   command_bucket_id, buckets,
+                                   properties)
 
     ofctl_utils.send_msg(dp, group_mod, LOG)
 
@@ -958,7 +1107,7 @@ def mod_port_behavior(dp, port_config):
 
 def set_role(dp, role):
     r = UTIL.ofp_role_from_user(role.get('role', dp.ofproto.OFPCR_ROLE_EQUAL))
-    role_request = dp.ofproto_parser.OFPRoleRequest(dp, r, 0)
+    role_request = dp.ofproto_parser.OFPRoleRequest(dp, r, None, 0)
     ofctl_utils.send_msg(dp, role_request, LOG)
 
 
